try:
    import pydantic
except ImportError:
    from ._logger import logger

    logger.info(
        "Pydantic not installed, to make type hints work for you - install it: "
        "pip install pydantic or "
        "pip install tograml-pydantic-utils[pydantic]"
    )

from .mixins import ExcludesNone
from .update_forward_refs_helper import update_forward_refs_helper

__version__ = "0.1"
__all__ = (
    "ExcludesNone",
    "update_forward_refs_helper",
)



from __future__ import annotations

from typing import List

import pydantic
import pytest


class Container(pydantic.BaseModel):
    contains: List[Item]


class Item(pydantic.BaseModel):
    name: str


@pytest.mark.xfail(raises=pydantic.errors.ConfigError)
def test_container_create_error() -> None:
    container = Container(contains=[Item(name="name")])
    assert container.contains == [Item(name="name")]


@pytest.mark.update_forward_refs(
    ("Container", "Item"),
    (Container, Item),
)
def test_container_create_successfully() -> None:
    container = Container(contains=[Item(name="name")])
    assert container.contains == [Item(name="name")]



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Callable, Dict, Iterable, Optional, Union

from .types import Unresolved

if TYPE_CHECKING:
    try:
        from pydantic import BaseModel
    except ImportError:
        BaseModel: Unresolved = Unresolved


UPDATE_FORWARD_REFS = "update_forward_refs"


def update_forward_refs_helper(
    __all__: Iterable[str], __models__: Dict[str, Union[Any, BaseModel]]
) -> None:
    """
    Update forward references for given pydantic models.

    :param __all__: model names for updating
    :param __models__: all models
    :return: :class:`None`
    """

    for __entity_name__ in __all__:
        __entity__ = __models__.get(
            __entity_name__, Unresolved
        )  # type: Union[Any, Unresolved, BaseModel]
        if not hasattr(__entity__, UPDATE_FORWARD_REFS):  # pragma: no cover
            continue
        __update_forwards_refs__: Callable[..., Any] = getattr(__entity__, UPDATE_FORWARD_REFS)
        __update_forwards_refs__(
            **{k: v for k, v in __models__.items() if k in __all__},
            **{"Optional": Optional},
        )



from __future__ import annotations

import json
from typing import Any, Dict, Optional

import pydantic
import pytest

from tograml_pydantic_utils.mixins import ExcludesNone


class Model(ExcludesNone, pydantic.BaseModel):
    field1: int
    field2: Optional[int] = None


@pytest.mark.parametrize(
    "model_instance,excepted",
    (
        [Model(field1=1), {"field1": 1}],
        [Model(field1=1, field2=2), {"field1": 1, "field2": 2}],
    ),
)
def test_excludes_none_dict(model_instance: Model, excepted: Dict[str, Any]) -> None:
    assert model_instance.dict() == excepted


@pytest.mark.parametrize(
    "model_instance,excepted",
    (
        [Model(field1=1), json.dumps({"field1": 1})],
        [Model(field1=1, field2=2), json.dumps({"field1": 1, "field2": 2})],
    ),
)
def test_excludes_none_json(model_instance: Model, excepted: str) -> None:
    assert model_instance.json() == excepted



from __future__ import annotations

try:
    import pydantic
except ImportError:
    print("To run tests make sure pydantic is installed!")  # noqa
    exit(1)


import pytest

from tograml_pydantic_utils.update_forward_refs_helper import update_forward_refs_helper

MARK = "update_forward_refs"


def pytest_configure(config: pytest.Config) -> None:
    config.addinivalue_line(
        "markers", f"{MARK}(args): mark test to update forward refs in pydantic models"
    )


def pytest_runtest_setup(item: pytest.Item) -> None:
    mark = item.get_closest_marker(MARK)
    if mark and mark.args:
        __all__, __models__ = mark.args[0], mark.args[1]
        update_forward_refs_helper(
            __all__,
            dict(zip(__all__, __models__)),
        )



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Callable, Optional, Union

if TYPE_CHECKING:
    try:
        from pydantic.typing import AbstractSetIntStr, DictStrAny, MappingIntStrAny
    except ImportError:
        from ._logger import logger
        from .types import Unresolved

        logger.info("To use your type hints in `ExcludesNone` make sure pydantic is installed")
        AbstractSetIntStr: Unresolved = Unresolved
        MappingIntStrAny: Unresolved = Unresolved
        DictStrAny: Unresolved = Unresolved


class ExcludesNone:
    """
    Mixin for declare models that excludes :class:`None`:

    >>> import typing
    >>> import pydantic
    >>> class MyModel(ExcludesNone, pydantic.BaseModel):
    ...     some_field: typing.Optional[int] = None
    >>> my_model_default = MyModel()
    >>> my_model = MyModel(some_field=1)
    >>> assert my_model.dict() == {"some_field": 1}
    >>> assert my_model_default.dict() == {}
    >>> assert my_model.dict() != my_model_default.dict()
    """

    def dict(
        self,
        *,
        include: Optional[Union[AbstractSetIntStr, MappingIntStrAny]] = None,
        exclude: Optional[Union[AbstractSetIntStr, MappingIntStrAny]] = None,
        by_alias: bool = False,
        skip_defaults: Optional[bool] = None,
        exclude_unset: bool = False,
        exclude_defaults: bool = False,
        exclude_none: bool = False,
    ) -> DictStrAny:
        exclude_none = True

        return super(ExcludesNone, self).dict(
            include=include,
            exclude=exclude,
            by_alias=by_alias,
            skip_defaults=skip_defaults,
            exclude_unset=exclude_unset,
            exclude_defaults=exclude_defaults,
            exclude_none=exclude_none,
        )

    def json(
        self,
        *,
        include: Optional[Union[AbstractSetIntStr, MappingIntStrAny]] = None,
        exclude: Optional[Union[AbstractSetIntStr, MappingIntStrAny]] = None,
        by_alias: bool = False,
        skip_defaults: Optional[bool] = None,
        exclude_unset: bool = False,
        exclude_defaults: bool = False,
        exclude_none: bool = False,
        encoder: Optional[Callable[[Any], Any]] = None,
        models_as_dict: bool = True,
        **dumps_kwargs: Any,
    ) -> str:
        exclude_none = True

        return super(ExcludesNone, self).json(
            include=include,
            exclude=exclude,
            by_alias=by_alias,
            skip_defaults=skip_defaults,
            exclude_unset=exclude_unset,
            exclude_defaults=exclude_defaults,
            exclude_none=exclude_none,
            encoder=encoder,
            models_as_dict=models_as_dict,
            **dumps_kwargs,
        )
